#include <windows.h>
#include <stdarg.h>
#include <iostream>
#include <sstream>
#include <fstream>
#pragma warning(disable:4996)
#include "tstring.h"
#pragma warning(default:4996)

// #include "porttime.h"
#include "NosuchDebug.h"

#include <list>
#include <vector>

using namespace std;

int NosuchDebugLevel = 0;    // We want debugging on when we start
bool NosuchDebugToConsole = true;
bool NosuchDebugToLog = false;
bool NosuchDebugAutoFlush = true;

// Normally this value is overwritten by the user of NosuchDebug
std::string NosuchDebugLogFile = "nosuch_debug.txt";
std::string NosuchDebugPrefix = "";

static std::list<std::string> DebugLog;
static bool NosuchDebugInitialized = false;
static HANDLE dMutex;

static void
NosuchDebugInit() {
	if ( ! NosuchDebugInitialized ) {
		dMutex = CreateMutex(NULL, false, NULL);
		NosuchDebugInitialized = true;
	}
}

static void
RealDebugDumpLog() {
	if (DebugLog.empty())
		return;
	ofstream f(NosuchDebugLogFile.c_str(),ios::app);
	while (!DebugLog.empty()) {
		std::string s = DebugLog.front();
	    f << s;
		DebugLog.pop_front();
	}
	f.close();
}

void
NosuchDebugDumpLog()
{
	NosuchDebugInit();
	DWORD wait = WaitForSingleObject( dMutex, INFINITE);
	if ( wait == WAIT_ABANDONED )
		return;

	RealDebugDumpLog();

	ReleaseMutex(dMutex);
}

void
NosuchErrorOutput(const char *fmt, ...) {

	if ( fmt == NULL ) {
		NosuchErrorOutput("fmt==NULL in NosuchErrorOutput!?\n");
		return;
	}

    va_list args;
    va_start(args, fmt);

    char msg[10000];
    vsprintf_s(msg,sizeof(msg)-2,fmt,args);
    va_end(args);

	char *p = strchr(msg,'\0');
	if ( p != NULL && p != msg && *(p-1) != '\n' ) {
		strcat_s(msg,sizeof(msg),"\n");
	}

	OutputDebugStringA(msg);
}

static void
RealNosuchDebug(int level, char const *fmt, va_list args)
{
	NosuchDebugInit();

	if ( level > NosuchDebugLevel )
		return;

	DWORD wait = WaitForSingleObject( dMutex, INFINITE);
	if ( wait == WAIT_ABANDONED )
		return;

	if ( fmt == NULL ) {
		fmt = "NULL pointer passed as fmt !?";
		NosuchErrorOutput("fmt==NULL in RealNosuchDebug\n");
	}

    // va_list args;
    char msg[10000];
	char* pmsg = msg;
	int msgsize = sizeof(msg)-2;

	if ( NosuchDebugPrefix != "" ) {
		int nchars = _snprintf_s(pmsg,msgsize,_TRUNCATE,"%s",NosuchDebugPrefix.c_str());
		pmsg += nchars;
		msgsize -= nchars;
	}

    // va_start(args, fmt);
    vsprintf_s(pmsg,msgsize,fmt,args);

	char *p = strchr(msg,'\0');
	if ( p != NULL && p != msg && *(p-1) != '\n' ) {
		strcat_s(msg,msgsize,"\n");
	}

	if ( NosuchDebugToConsole || level == ERROR_OUTPUT ) {
		OutputDebugStringA(msg);
	}
	if ( NosuchDebugToLog ) {
		DebugLog.push_back(msg);
		if ( NosuchDebugAutoFlush )
			RealDebugDumpLog();
	}

    // va_end(args);

	ReleaseMutex(dMutex);
}

void
NosuchDebug(char const *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
	RealNosuchDebug(0,fmt,args);
    va_end(args);
}

void
NosuchDebug(int level, char const *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
	RealNosuchDebug(level,fmt,args);
    va_end(args);
}

void
NosuchDebug(std::string s)
{
	NosuchDebug("%s",s.c_str());
}

void
NosuchDebug(int level, std::string s)
{
	NosuchDebug(level,"%s",s.c_str());
}

