#include "NosuchUtil.h"
#include <iostream>
#include <fstream>
#include <string>
#include <algorithm>
#include <vector>
#include <iterator>

using namespace std;

#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#ifdef NOSUCH_LOCK
#include <pthread.h>

void NosuchLockInit(pthread_mutex_t* mutex, char *tag) {

	*mutex = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER;
	// *mutex = PTHREAD_MUTEX_INITIALIZER;

	// NosuchDebug("NosuchLockInit pthread=%d tag=%s mutex=%d",(int)pthread_self().p,tag,(int)mutex);
}

void NosuchLock(pthread_mutex_t* mutex, char *tag) {
	// NosuchDebug("NosuchLock pthread=%d tag=%s mutex=%d",(int)pthread_self().p,tag,(int)mutex);
	int r = pthread_mutex_lock(mutex);
	if ( r == EDEADLK ) {
		NosuchDebug(ERROR_OUTPUT,"Hey! NosuchLock detects DEADLOCK!!  tag=%s",tag);
	} else if ( r != 0 ) {
		NosuchDebug(ERROR_OUTPUT,"Hey! pthread_mutex_lock tag=%s returned non-zero!  r=%d",tag,r);
	}
}

void NosuchUnlock(pthread_mutex_t* mutex, char *tag) {
	// NosuchDebug("NosuchUnlock pthread=%d tag=%s mutex=%d",(int)pthread_self().p,tag,(int)mutex);
	int r = pthread_mutex_unlock(mutex);
	if ( r != 0 ) {
		NosuchDebug(ERROR_OUTPUT,"Hey! pthread_mutex_unlock tag=%s returned non-zero!  r=%d",tag,r);
	}
}
#endif

std::string
NosuchSnprintf(const char *fmt, ...)
{
	static char *msg = NULL;
	static int msglen = 4096;
	va_list args;

	if ( msg == NULL )
		msg = (char*)malloc(msglen);

	while (1) {
		va_start(args, fmt);
		int written = vsnprintf_s(msg,msglen,_TRUNCATE,fmt,args);
		va_end(args);
		if ( written < msglen ) {
			return std::string(msg);
		}
		free(msg);
		msglen *= 2;
		msg = (char*)malloc(msglen);
	}
}

std::vector<std::string> NosuchSplitOnAnyChar(std::string s, std::string sepchars)
{
	std::vector<std::string> result;
	const char *seps = sepchars.c_str();
	const char *str = s.c_str();

    while(1) {
        const char *begin = str;
		while( strchr(seps,*str) == NULL && *str) {
                str++;
		}
        result.push_back(std::string(begin, str));
		if(0 == *str) {
			break;
		}
		// skip one or more of the sepchars
		while( strchr(seps,*str) != NULL && *str ) {
			str++;
		}
    }
    return result;
}

std::vector<std::string> NosuchSplitOnString(const std::string& s, const std::string& delim, const bool keep_empty = true) {
	std::vector<std::string> result;
    if (delim.empty()) {
        result.push_back(s);
        return result;
    }
    std::string::const_iterator substart = s.begin(), subend;
    while (true) {
        subend = search(substart, s.end(), delim.begin(), delim.end());
		std::string temp(substart, subend);
        if (keep_empty || !temp.empty()) {
            result.push_back(temp);
        }
        if (subend == s.end()) {
            break;
        }
        substart = subend + delim.size();
    }
    return result;
}

